;==============================================================================================
; NEO22v5: modified program of the NEO22 running on v5-boards
; source based on the NEO22 disassembly by cableiso (2/2/2)
;----------------------------------------------------------------------------------------------
; Processor: Microchip PIC 12C508A
;----------------------------------------------------------------------------------------------
; Press the PowerOn/Reset-Button short (below 0.6 seconds) to switch off chip-funtions (mode1).
; Turn on the PS2 by pressing Eject enables operation mode 2. This means each time you insert
; any cd the chip sends SCEx-messages for some time. Holding the button longer than 1 second
; on power-on turns the chip into mode 3. In this operation mode we are continously sending
; SCEx-messages 'till the button is released.
; 
; Put the pins 1,2,4,5,6,8 to the same places NEO22 would sit down. I placed a 1F cap at pin 2,
; negative side pointing to motherboard. Other pins can point to any places. (eg. if u have all
; wires already setup for neo22, just let them be there, only replace the pic)
; If u do a new installation and dont want to solder pins 5 and 6, just wire them to pin 8 on
; the pic. Note: operation mode 2 wont work then.
;----------------------------------------------------------------------------------------------
; Pinout Description:
;
;                     PIC 12C508A
;                   /---\/---\
; (+5V    )    Vdd -|1     8|- Vss    (GND    )
; (Data   ) << GP5 -|2      7|- GP0    (na     )
; (na     )    GP4 -|3      6|- GP1 << (CdOpen )
; (Reset  ) >> GP3 -|4      5|- GP2 << (CdClose)
;                   \--------/
;==============================================================================================


;==============================================================================================
; Fuses & Compiler Settings
;==============================================================================================

                __FUSES _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC
                list p=12C508A
                #include <p12C508a.inc>


;==============================================================================================
; Variables
;==============================================================================================

t0              equ     0x10
t1              equ     0x11
NextLetter      equ     0x12
OfsLetter       equ     0x13
BitCounter      equ     0x14
PhraseRpCntr    equ     0x15
LetterCntr      equ     0x16
NumPhrases      equ     0x17


;==============================================================================================
; Code
;==============================================================================================

                ; ---Program Entry Point---
                movwf   OSCCAL
                goto    Main
                dt      "NEO22v5-1"


;==============================================================================================
; main routine
;==============================================================================================
Main

                ; ---initialize---
                clrf    t0              ; Clear a few variables
                clrf    t1
                clrf    NextLetter
                clrf    BitCounter
                clrf    LetterCntr
                clrf    NumPhrases

                clrf    GPIO            ; Set all port buffers to 0
                movlw   0xC7            ; Set chip configuration to
                option                  ; TMR0 internal, pullups off
                movlw   0xFF            ; Set all I/O pins as 
                tris    GPIO            ; input/tristate

ReadyToSendLp: ;btfss   GPIO,0x5        ; Loop here until GP5 (pin 2)
               ;goto    ReadyToSendLp   ; goes HIGH (why?)

                ; ---check reset-button state to choose operation mode---
Wait4ResetDown: btfsc   GPIO,0x1        ; if GP1 (pin 6) (=> cd-tray open) is HIGH,
                goto    OperationMode2  ; then ps2 was turned on by pressing Eject-Button
                btfsc   GPIO,0x3        ; Loop here until GP3 (pin 4) (=> Reset-Button)
                goto    Wait4ResetDown  ; goes LOW (is pressed)

                movlw   0x14            ; Delay for .6 sec (20 * 32.778ms)
                call    TimerDelay

                btfsc   GPIO,0x3        ; If Reset-Button is released before .6 sec
                goto    OperationMode1  ; we start mode 1

Wait4ResetUp:   btfss   GPIO,0x3        ; Loop until Reset-Button is released
                goto    Wait4ResetUp

                goto    OperationMode3  ; now we start mode 3


;==============================================================================================
; OperationMode1: deactivate chip
;----------------------------------------------------------------------------------------------
; Called by: Main
;==============================================================================================
OperationMode1

                nop
                goto    OperationMode1


;==============================================================================================
; OperationMode2: only send some phrases if try closes
;----------------------------------------------------------------------------------------------
; Called by: Main
;==============================================================================================
OperationMode2

                btfss   GPIO,0x2        ; if GP2 (pin 5) (=> cd-tray close) is LOW,
                goto    OperationMode2  ; then loop untill cd tray closes

                movlw   0x14            ; send 20*3 phrases (need some lower value like 0x10?)
                clrf    PhraseRpCntr
                movwf   PhraseRpCntr    ; Store # into PhraseRpCntr

om2_repeat:     movlw   0xF             ; Delay for about 1/2 sec (15 x 32.778 = 491ms)
                call    TimerDelay
                movlw   0x3             ; send one phrase
                call    SendCopyright

                decfsz  PhraseRpCntr, F ; Decrement PhraseRpCntr
                goto    om2_repeat

                goto    OperationMode2


;==============================================================================================
; OperationMode3: send continous phrases (except if the try is open)
;----------------------------------------------------------------------------------------------
; Called by: Main
;==============================================================================================
OperationMode3

                btfss   GPIO,0x1        ; if GP1 (pin 6) (=> cd-tray open) is LOW,
                goto    om3_send        ; then loop untill cd tray closes
om3_wait:       btfss   GPIO,0x2        ; if GP2 (pin 5) (=> cd-tray close) is LOW,
                goto    om3_wait        ; then loop untill cd tray closes

om3_send:       movlw   0xF             ; Delay for about 1/2 sec (15 x 32.778 = 491ms)
                call    TimerDelay
                movlw   0x3             ; send 3 phrases
                call    SendCopyright
                goto    OperationMode3


;==============================================================================================
; SendCopyright: Send multiple "SCEx" messages to the ps2.  Number of phrases to send is 
; passed in W register
;----------------------------------------------------------------------------------------------
; Called by: OperationModeX
;==============================================================================================
SendCopyright

                movwf   NumPhrases      ; save our number of phrases to send

L14:            movlw   0x4A            ; delay about 75ms (74 * 1.01ms)
                call    ShortDelay
                clrf    OfsLetter       ; start with the first letter in our phrase
                movlw   0x4             ; load our counter into LetterCntr 
                movwf   LetterCntr      ; (4 letters per phrase)

L15:            movf    OfsLetter,W     ; Restore index
                call    Phrases         ; Go get next letter of copyright message in W
                movwf   NextLetter      ; Move that letter to NextLetter
                comf    NextLetter, F   ; And invert the bits
                movlw   0x8             ; 8 bits in a byte, here we
                movwf   BitCounter      ; load up our bit counter
                movlw   0xFF            ; Tristate all port pins
                tris    GPIO
                movlw   0x4             ; delay 4ms
                call    ShortDelay

L16:            rrf     NextLetter, F   ; Roll a bit out of buffer & into the Carry flag
                bcf     0x6,0x5         ; Set GP5 buffer to LOW
                btfsc   STATUS,C        ; If carry flag is low, 
                movlw   0xFF            ; We'll tristate all ports 
                btfss   STATUS,C        ; If carry flag is high
                movlw   0xDF            ; Set GP5 as an output (it'll be low)
                tris    GPIO            ; Set ports
                movlw   0x4             ; load up a 4ms delay
                call    ShortDelay
                decfsz  BitCounter, F   ; Decrement bit counter & continue sending
                goto    L16             ; if bit counter != 0
                movlw   0xDF            ; Else fire up some stop bits..
                tris    GPIO            ; Set GP5 as an output low
                movlw   0x8             ; Load up an 8ms delay
                call    ShortDelay
                incf    OfsLetter, F    ; Increment our phrase index
                decfsz  LetterCntr, F   ; decrement letter counter - All letters sent?
                goto    L15             ; If not, go do next letter
                decfsz  NumPhrases, F   ; If so, decrement phrase counter - All phrases sent?
                goto    L14             ; If not, go send next phrase

                movlw   0xFF            ; Tristate all port pins
                tris    GPIO

                retlw   0x3             ; If all phrases done, return to caller


;==============================================================================================
; TimerDelay: Waste (W * 32.778)ms
;----------------------------------------------------------------------------------------------
; Called by: Start, OperationModeX
;==============================================================================================
TimerDelay

                movwf   t0
td_loop:        clrf    TMR0            ; Clear the Timer
td_int_loop:    btfss   TMR0,0x7        ; Loop here until timer > 127
                goto    td_int_loop
                decfsz  t0, F           ; Decrement t0 
                goto    td_loop         ; Loop again if necessary
                retlw   0x0             ; else return


;==============================================================================================
; ShortDelay: Delay for W milliseconds. Actual delay time is about 1.01ms so for large values
; of W (W > 50) take this into consideration
;----------------------------------------------------------------------------------------------
; Called by: SendCopyright
;==============================================================================================
ShortDelay

                movwf   t0              ; Store # of ms into t0
sd_loop:        movlw   0xFA            ; Load up a counter w/ 250
                movwf   t1             
sd_int_loop:    nop                     ; These couple instructions take a 
                decfsz  t1, F           ; total of 4us each time, so
                goto    sd_int_loop     ; that's where we get our 1ms
                decfsz  t0, F           ; More milliseconds or are we done?
                goto    sd_loop         ; If more time, repeat the cycle
                retlw   0x0             ; Else return to caller


;==============================================================================================
; Phrases: Return letters of the sony messages for transmission to the ps2
;----------------------------------------------------------------------------------------------
; Called by: SendCopyright
;==============================================================================================
Phrases

                addwf   PCL, F          ; Add w to PC & return that letter
                dt      "SCEE"          ; change SCEE to SCEA(US)/SCEI(JP) for diff. regions

              
wiRe            dt      "wiRe"          ; That's it
                end
